# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['TransitRouterRouteTableArgs', 'TransitRouterRouteTable']

@pulumi.input_type
class TransitRouterRouteTableArgs:
    def __init__(__self__, *,
                 transit_router_id: pulumi.Input[str],
                 dry_run: Optional[pulumi.Input[bool]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 transit_router_route_table_description: Optional[pulumi.Input[str]] = None,
                 transit_router_route_table_name: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a TransitRouterRouteTable resource.
        :param pulumi.Input[str] transit_router_id: The ID of the transit router.
        :param pulumi.Input[bool] dry_run: The dry run.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: A mapping of tags to assign to the resource.
        :param pulumi.Input[str] transit_router_route_table_description: The description of the transit router route table.
        :param pulumi.Input[str] transit_router_route_table_name: The name of the transit router route table.
        """
        pulumi.set(__self__, "transit_router_id", transit_router_id)
        if dry_run is not None:
            pulumi.set(__self__, "dry_run", dry_run)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if transit_router_route_table_description is not None:
            pulumi.set(__self__, "transit_router_route_table_description", transit_router_route_table_description)
        if transit_router_route_table_name is not None:
            pulumi.set(__self__, "transit_router_route_table_name", transit_router_route_table_name)

    @property
    @pulumi.getter(name="transitRouterId")
    def transit_router_id(self) -> pulumi.Input[str]:
        """
        The ID of the transit router.
        """
        return pulumi.get(self, "transit_router_id")

    @transit_router_id.setter
    def transit_router_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "transit_router_id", value)

    @property
    @pulumi.getter(name="dryRun")
    def dry_run(self) -> Optional[pulumi.Input[bool]]:
        """
        The dry run.
        """
        return pulumi.get(self, "dry_run")

    @dry_run.setter
    def dry_run(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "dry_run", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        A mapping of tags to assign to the resource.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)

    @property
    @pulumi.getter(name="transitRouterRouteTableDescription")
    def transit_router_route_table_description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the transit router route table.
        """
        return pulumi.get(self, "transit_router_route_table_description")

    @transit_router_route_table_description.setter
    def transit_router_route_table_description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "transit_router_route_table_description", value)

    @property
    @pulumi.getter(name="transitRouterRouteTableName")
    def transit_router_route_table_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the transit router route table.
        """
        return pulumi.get(self, "transit_router_route_table_name")

    @transit_router_route_table_name.setter
    def transit_router_route_table_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "transit_router_route_table_name", value)


@pulumi.input_type
class _TransitRouterRouteTableState:
    def __init__(__self__, *,
                 dry_run: Optional[pulumi.Input[bool]] = None,
                 status: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 transit_router_id: Optional[pulumi.Input[str]] = None,
                 transit_router_route_table_description: Optional[pulumi.Input[str]] = None,
                 transit_router_route_table_id: Optional[pulumi.Input[str]] = None,
                 transit_router_route_table_name: Optional[pulumi.Input[str]] = None,
                 transit_router_route_table_type: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering TransitRouterRouteTable resources.
        :param pulumi.Input[bool] dry_run: The dry run.
        :param pulumi.Input[str] status: The associating status of the Transit Router.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: A mapping of tags to assign to the resource.
        :param pulumi.Input[str] transit_router_id: The ID of the transit router.
        :param pulumi.Input[str] transit_router_route_table_description: The description of the transit router route table.
        :param pulumi.Input[str] transit_router_route_table_id: The id of the transit router route table.
        :param pulumi.Input[str] transit_router_route_table_name: The name of the transit router route table.
        :param pulumi.Input[str] transit_router_route_table_type: The type of the transit router route table. Valid values: `Custom`, `System`.
        """
        if dry_run is not None:
            pulumi.set(__self__, "dry_run", dry_run)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if transit_router_id is not None:
            pulumi.set(__self__, "transit_router_id", transit_router_id)
        if transit_router_route_table_description is not None:
            pulumi.set(__self__, "transit_router_route_table_description", transit_router_route_table_description)
        if transit_router_route_table_id is not None:
            pulumi.set(__self__, "transit_router_route_table_id", transit_router_route_table_id)
        if transit_router_route_table_name is not None:
            pulumi.set(__self__, "transit_router_route_table_name", transit_router_route_table_name)
        if transit_router_route_table_type is not None:
            pulumi.set(__self__, "transit_router_route_table_type", transit_router_route_table_type)

    @property
    @pulumi.getter(name="dryRun")
    def dry_run(self) -> Optional[pulumi.Input[bool]]:
        """
        The dry run.
        """
        return pulumi.get(self, "dry_run")

    @dry_run.setter
    def dry_run(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "dry_run", value)

    @property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[str]]:
        """
        The associating status of the Transit Router.
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "status", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        A mapping of tags to assign to the resource.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)

    @property
    @pulumi.getter(name="transitRouterId")
    def transit_router_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the transit router.
        """
        return pulumi.get(self, "transit_router_id")

    @transit_router_id.setter
    def transit_router_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "transit_router_id", value)

    @property
    @pulumi.getter(name="transitRouterRouteTableDescription")
    def transit_router_route_table_description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the transit router route table.
        """
        return pulumi.get(self, "transit_router_route_table_description")

    @transit_router_route_table_description.setter
    def transit_router_route_table_description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "transit_router_route_table_description", value)

    @property
    @pulumi.getter(name="transitRouterRouteTableId")
    def transit_router_route_table_id(self) -> Optional[pulumi.Input[str]]:
        """
        The id of the transit router route table.
        """
        return pulumi.get(self, "transit_router_route_table_id")

    @transit_router_route_table_id.setter
    def transit_router_route_table_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "transit_router_route_table_id", value)

    @property
    @pulumi.getter(name="transitRouterRouteTableName")
    def transit_router_route_table_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the transit router route table.
        """
        return pulumi.get(self, "transit_router_route_table_name")

    @transit_router_route_table_name.setter
    def transit_router_route_table_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "transit_router_route_table_name", value)

    @property
    @pulumi.getter(name="transitRouterRouteTableType")
    def transit_router_route_table_type(self) -> Optional[pulumi.Input[str]]:
        """
        The type of the transit router route table. Valid values: `Custom`, `System`.
        """
        return pulumi.get(self, "transit_router_route_table_type")

    @transit_router_route_table_type.setter
    def transit_router_route_table_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "transit_router_route_table_type", value)


class TransitRouterRouteTable(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 dry_run: Optional[pulumi.Input[bool]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 transit_router_id: Optional[pulumi.Input[str]] = None,
                 transit_router_route_table_description: Optional[pulumi.Input[str]] = None,
                 transit_router_route_table_name: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Provides a CEN transit router route table resource.[What is Cen Transit Router Route Table](https://www.alibabacloud.com/help/en/cen/developer-reference/api-cbn-2017-09-12-createtransitrouterroutetable)

        > **NOTE:** Available since v1.126.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        example = alicloud.cen.Instance("example",
            cen_instance_name="tf_example",
            description="an example for cen")
        example_transit_router = alicloud.cen.TransitRouter("example",
            transit_router_name="tf_example",
            cen_id=example.id)
        example_transit_router_route_table = alicloud.cen.TransitRouterRouteTable("example", transit_router_id=example_transit_router.transit_router_id)
        ```

        ## Import

        CEN transit router route table  can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:cen/transitRouterRouteTable:TransitRouterRouteTable default tr-*********:vtb-********
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] dry_run: The dry run.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: A mapping of tags to assign to the resource.
        :param pulumi.Input[str] transit_router_id: The ID of the transit router.
        :param pulumi.Input[str] transit_router_route_table_description: The description of the transit router route table.
        :param pulumi.Input[str] transit_router_route_table_name: The name of the transit router route table.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: TransitRouterRouteTableArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a CEN transit router route table resource.[What is Cen Transit Router Route Table](https://www.alibabacloud.com/help/en/cen/developer-reference/api-cbn-2017-09-12-createtransitrouterroutetable)

        > **NOTE:** Available since v1.126.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        example = alicloud.cen.Instance("example",
            cen_instance_name="tf_example",
            description="an example for cen")
        example_transit_router = alicloud.cen.TransitRouter("example",
            transit_router_name="tf_example",
            cen_id=example.id)
        example_transit_router_route_table = alicloud.cen.TransitRouterRouteTable("example", transit_router_id=example_transit_router.transit_router_id)
        ```

        ## Import

        CEN transit router route table  can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:cen/transitRouterRouteTable:TransitRouterRouteTable default tr-*********:vtb-********
        ```

        :param str resource_name: The name of the resource.
        :param TransitRouterRouteTableArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(TransitRouterRouteTableArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 dry_run: Optional[pulumi.Input[bool]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 transit_router_id: Optional[pulumi.Input[str]] = None,
                 transit_router_route_table_description: Optional[pulumi.Input[str]] = None,
                 transit_router_route_table_name: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = TransitRouterRouteTableArgs.__new__(TransitRouterRouteTableArgs)

            __props__.__dict__["dry_run"] = dry_run
            __props__.__dict__["tags"] = tags
            if transit_router_id is None and not opts.urn:
                raise TypeError("Missing required property 'transit_router_id'")
            __props__.__dict__["transit_router_id"] = transit_router_id
            __props__.__dict__["transit_router_route_table_description"] = transit_router_route_table_description
            __props__.__dict__["transit_router_route_table_name"] = transit_router_route_table_name
            __props__.__dict__["status"] = None
            __props__.__dict__["transit_router_route_table_id"] = None
            __props__.__dict__["transit_router_route_table_type"] = None
        super(TransitRouterRouteTable, __self__).__init__(
            'alicloud:cen/transitRouterRouteTable:TransitRouterRouteTable',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            dry_run: Optional[pulumi.Input[bool]] = None,
            status: Optional[pulumi.Input[str]] = None,
            tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            transit_router_id: Optional[pulumi.Input[str]] = None,
            transit_router_route_table_description: Optional[pulumi.Input[str]] = None,
            transit_router_route_table_id: Optional[pulumi.Input[str]] = None,
            transit_router_route_table_name: Optional[pulumi.Input[str]] = None,
            transit_router_route_table_type: Optional[pulumi.Input[str]] = None) -> 'TransitRouterRouteTable':
        """
        Get an existing TransitRouterRouteTable resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[bool] dry_run: The dry run.
        :param pulumi.Input[str] status: The associating status of the Transit Router.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: A mapping of tags to assign to the resource.
        :param pulumi.Input[str] transit_router_id: The ID of the transit router.
        :param pulumi.Input[str] transit_router_route_table_description: The description of the transit router route table.
        :param pulumi.Input[str] transit_router_route_table_id: The id of the transit router route table.
        :param pulumi.Input[str] transit_router_route_table_name: The name of the transit router route table.
        :param pulumi.Input[str] transit_router_route_table_type: The type of the transit router route table. Valid values: `Custom`, `System`.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _TransitRouterRouteTableState.__new__(_TransitRouterRouteTableState)

        __props__.__dict__["dry_run"] = dry_run
        __props__.__dict__["status"] = status
        __props__.__dict__["tags"] = tags
        __props__.__dict__["transit_router_id"] = transit_router_id
        __props__.__dict__["transit_router_route_table_description"] = transit_router_route_table_description
        __props__.__dict__["transit_router_route_table_id"] = transit_router_route_table_id
        __props__.__dict__["transit_router_route_table_name"] = transit_router_route_table_name
        __props__.__dict__["transit_router_route_table_type"] = transit_router_route_table_type
        return TransitRouterRouteTable(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="dryRun")
    def dry_run(self) -> pulumi.Output[Optional[bool]]:
        """
        The dry run.
        """
        return pulumi.get(self, "dry_run")

    @property
    @pulumi.getter
    def status(self) -> pulumi.Output[str]:
        """
        The associating status of the Transit Router.
        """
        return pulumi.get(self, "status")

    @property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        A mapping of tags to assign to the resource.
        """
        return pulumi.get(self, "tags")

    @property
    @pulumi.getter(name="transitRouterId")
    def transit_router_id(self) -> pulumi.Output[str]:
        """
        The ID of the transit router.
        """
        return pulumi.get(self, "transit_router_id")

    @property
    @pulumi.getter(name="transitRouterRouteTableDescription")
    def transit_router_route_table_description(self) -> pulumi.Output[str]:
        """
        The description of the transit router route table.
        """
        return pulumi.get(self, "transit_router_route_table_description")

    @property
    @pulumi.getter(name="transitRouterRouteTableId")
    def transit_router_route_table_id(self) -> pulumi.Output[str]:
        """
        The id of the transit router route table.
        """
        return pulumi.get(self, "transit_router_route_table_id")

    @property
    @pulumi.getter(name="transitRouterRouteTableName")
    def transit_router_route_table_name(self) -> pulumi.Output[Optional[str]]:
        """
        The name of the transit router route table.
        """
        return pulumi.get(self, "transit_router_route_table_name")

    @property
    @pulumi.getter(name="transitRouterRouteTableType")
    def transit_router_route_table_type(self) -> pulumi.Output[str]:
        """
        The type of the transit router route table. Valid values: `Custom`, `System`.
        """
        return pulumi.get(self, "transit_router_route_table_type")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = [
    'DbClusterDbClusterAccessWhiteListArgs',
    'DbClusterDbClusterAccessWhiteListArgsDict',
    'DbClusterMultiZoneVswitchListArgs',
    'DbClusterMultiZoneVswitchListArgsDict',
]

MYPY = False

if not MYPY:
    class DbClusterDbClusterAccessWhiteListArgsDict(TypedDict):
        db_cluster_ip_array_attribute: NotRequired[pulumi.Input[str]]
        """
        Field `db_cluster_ip_array_attribute` has been removed from provider.
        """
        db_cluster_ip_array_name: NotRequired[pulumi.Input[str]]
        """
        Whitelist group name.
        """
        security_ip_list: NotRequired[pulumi.Input[str]]
        """
        The IP address list under the whitelist group.
        """
elif False:
    DbClusterDbClusterAccessWhiteListArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class DbClusterDbClusterAccessWhiteListArgs:
    def __init__(__self__, *,
                 db_cluster_ip_array_attribute: Optional[pulumi.Input[str]] = None,
                 db_cluster_ip_array_name: Optional[pulumi.Input[str]] = None,
                 security_ip_list: Optional[pulumi.Input[str]] = None):
        """
        :param pulumi.Input[str] db_cluster_ip_array_attribute: Field `db_cluster_ip_array_attribute` has been removed from provider.
        :param pulumi.Input[str] db_cluster_ip_array_name: Whitelist group name.
        :param pulumi.Input[str] security_ip_list: The IP address list under the whitelist group.
        """
        if db_cluster_ip_array_attribute is not None:
            pulumi.set(__self__, "db_cluster_ip_array_attribute", db_cluster_ip_array_attribute)
        if db_cluster_ip_array_name is not None:
            pulumi.set(__self__, "db_cluster_ip_array_name", db_cluster_ip_array_name)
        if security_ip_list is not None:
            pulumi.set(__self__, "security_ip_list", security_ip_list)

    @property
    @pulumi.getter(name="dbClusterIpArrayAttribute")
    def db_cluster_ip_array_attribute(self) -> Optional[pulumi.Input[str]]:
        """
        Field `db_cluster_ip_array_attribute` has been removed from provider.
        """
        return pulumi.get(self, "db_cluster_ip_array_attribute")

    @db_cluster_ip_array_attribute.setter
    def db_cluster_ip_array_attribute(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "db_cluster_ip_array_attribute", value)

    @property
    @pulumi.getter(name="dbClusterIpArrayName")
    def db_cluster_ip_array_name(self) -> Optional[pulumi.Input[str]]:
        """
        Whitelist group name.
        """
        return pulumi.get(self, "db_cluster_ip_array_name")

    @db_cluster_ip_array_name.setter
    def db_cluster_ip_array_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "db_cluster_ip_array_name", value)

    @property
    @pulumi.getter(name="securityIpList")
    def security_ip_list(self) -> Optional[pulumi.Input[str]]:
        """
        The IP address list under the whitelist group.
        """
        return pulumi.get(self, "security_ip_list")

    @security_ip_list.setter
    def security_ip_list(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "security_ip_list", value)


if not MYPY:
    class DbClusterMultiZoneVswitchListArgsDict(TypedDict):
        vswitch_id: pulumi.Input[str]
        """
        The ID of the vswitch.
        """
        zone_id: NotRequired[pulumi.Input[str]]
        """
        The zone ID of the vswitch.
        """
elif False:
    DbClusterMultiZoneVswitchListArgsDict: TypeAlias = Mapping[str, Any]

@pulumi.input_type
class DbClusterMultiZoneVswitchListArgs:
    def __init__(__self__, *,
                 vswitch_id: pulumi.Input[str],
                 zone_id: Optional[pulumi.Input[str]] = None):
        """
        :param pulumi.Input[str] vswitch_id: The ID of the vswitch.
        :param pulumi.Input[str] zone_id: The zone ID of the vswitch.
        """
        pulumi.set(__self__, "vswitch_id", vswitch_id)
        if zone_id is not None:
            pulumi.set(__self__, "zone_id", zone_id)

    @property
    @pulumi.getter(name="vswitchId")
    def vswitch_id(self) -> pulumi.Input[str]:
        """
        The ID of the vswitch.
        """
        return pulumi.get(self, "vswitch_id")

    @vswitch_id.setter
    def vswitch_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "vswitch_id", value)

    @property
    @pulumi.getter(name="zoneId")
    def zone_id(self) -> Optional[pulumi.Input[str]]:
        """
        The zone ID of the vswitch.
        """
        return pulumi.get(self, "zone_id")

    @zone_id.setter
    def zone_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "zone_id", value)





# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs

__all__ = [
    'GetEcsLaunchTemplatesResult',
    'AwaitableGetEcsLaunchTemplatesResult',
    'get_ecs_launch_templates',
    'get_ecs_launch_templates_output',
]

@pulumi.output_type
class GetEcsLaunchTemplatesResult:
    """
    A collection of values returned by getEcsLaunchTemplates.
    """
    def __init__(__self__, enable_details=None, id=None, ids=None, launch_template_name=None, name_regex=None, names=None, output_file=None, template_resource_group_id=None, template_tags=None, templates=None):
        if enable_details and not isinstance(enable_details, bool):
            raise TypeError("Expected argument 'enable_details' to be a bool")
        pulumi.set(__self__, "enable_details", enable_details)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ids and not isinstance(ids, list):
            raise TypeError("Expected argument 'ids' to be a list")
        pulumi.set(__self__, "ids", ids)
        if launch_template_name and not isinstance(launch_template_name, str):
            raise TypeError("Expected argument 'launch_template_name' to be a str")
        pulumi.set(__self__, "launch_template_name", launch_template_name)
        if name_regex and not isinstance(name_regex, str):
            raise TypeError("Expected argument 'name_regex' to be a str")
        pulumi.set(__self__, "name_regex", name_regex)
        if names and not isinstance(names, list):
            raise TypeError("Expected argument 'names' to be a list")
        pulumi.set(__self__, "names", names)
        if output_file and not isinstance(output_file, str):
            raise TypeError("Expected argument 'output_file' to be a str")
        pulumi.set(__self__, "output_file", output_file)
        if template_resource_group_id and not isinstance(template_resource_group_id, str):
            raise TypeError("Expected argument 'template_resource_group_id' to be a str")
        pulumi.set(__self__, "template_resource_group_id", template_resource_group_id)
        if template_tags and not isinstance(template_tags, dict):
            raise TypeError("Expected argument 'template_tags' to be a dict")
        pulumi.set(__self__, "template_tags", template_tags)
        if templates and not isinstance(templates, list):
            raise TypeError("Expected argument 'templates' to be a list")
        pulumi.set(__self__, "templates", templates)

    @property
    @pulumi.getter(name="enableDetails")
    def enable_details(self) -> Optional[bool]:
        return pulumi.get(self, "enable_details")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def ids(self) -> Sequence[str]:
        return pulumi.get(self, "ids")

    @property
    @pulumi.getter(name="launchTemplateName")
    def launch_template_name(self) -> Optional[str]:
        """
        The Launch Template Name.
        """
        return pulumi.get(self, "launch_template_name")

    @property
    @pulumi.getter(name="nameRegex")
    def name_regex(self) -> Optional[str]:
        return pulumi.get(self, "name_regex")

    @property
    @pulumi.getter
    def names(self) -> Sequence[str]:
        """
        A list of Launch Template names.
        """
        return pulumi.get(self, "names")

    @property
    @pulumi.getter(name="outputFile")
    def output_file(self) -> Optional[str]:
        return pulumi.get(self, "output_file")

    @property
    @pulumi.getter(name="templateResourceGroupId")
    def template_resource_group_id(self) -> Optional[str]:
        return pulumi.get(self, "template_resource_group_id")

    @property
    @pulumi.getter(name="templateTags")
    def template_tags(self) -> Optional[Mapping[str, str]]:
        """
        The template tags.
        """
        return pulumi.get(self, "template_tags")

    @property
    @pulumi.getter
    def templates(self) -> Sequence['outputs.GetEcsLaunchTemplatesTemplateResult']:
        """
        A list of Ecs Launch Templates. Each element contains the following attributes:
        """
        return pulumi.get(self, "templates")


class AwaitableGetEcsLaunchTemplatesResult(GetEcsLaunchTemplatesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetEcsLaunchTemplatesResult(
            enable_details=self.enable_details,
            id=self.id,
            ids=self.ids,
            launch_template_name=self.launch_template_name,
            name_regex=self.name_regex,
            names=self.names,
            output_file=self.output_file,
            template_resource_group_id=self.template_resource_group_id,
            template_tags=self.template_tags,
            templates=self.templates)


def get_ecs_launch_templates(enable_details: Optional[bool] = None,
                             ids: Optional[Sequence[str]] = None,
                             launch_template_name: Optional[str] = None,
                             name_regex: Optional[str] = None,
                             output_file: Optional[str] = None,
                             template_resource_group_id: Optional[str] = None,
                             template_tags: Optional[Mapping[str, str]] = None,
                             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetEcsLaunchTemplatesResult:
    """
    This data source provides the Ecs Launch Templates of the current Alibaba Cloud user.

    > **NOTE:** Available since v1.120.0.

    ## Example Usage

    Basic Usage

    ```python
    import pulumi
    import pulumi_alicloud as alicloud

    default = alicloud.get_zones(available_disk_category="cloud_efficiency",
        available_resource_creation="VSwitch")
    default_get_instance_types = alicloud.ecs.get_instance_types(availability_zone=default.zones[0].id)
    default_get_images = alicloud.ecs.get_images(name_regex="^ubuntu_18.*64",
        owners="system")
    default_network = alicloud.vpc.Network("default",
        vpc_name="terraform-example",
        cidr_block="172.17.3.0/24")
    default_switch = alicloud.vpc.Switch("default",
        vswitch_name="terraform-example",
        cidr_block="172.17.3.0/24",
        vpc_id=default_network.id,
        zone_id=default.zones[0].id)
    default_security_group = alicloud.ecs.SecurityGroup("default",
        name="terraform-example",
        vpc_id=default_network.id)
    default_ecs_launch_template = alicloud.ecs.EcsLaunchTemplate("default",
        launch_template_name="terraform-example",
        description="terraform-example",
        image_id=default_get_images.images[0].id,
        host_name="terraform-example",
        instance_charge_type="PrePaid",
        instance_name="terraform-example",
        instance_type=default_get_instance_types.instance_types[0].id,
        internet_charge_type="PayByBandwidth",
        internet_max_bandwidth_in=5,
        internet_max_bandwidth_out=5,
        io_optimized="optimized",
        key_pair_name="key_pair_name",
        ram_role_name="ram_role_name",
        network_type="vpc",
        security_enhancement_strategy="Active",
        spot_price_limit=5,
        spot_strategy="SpotWithPriceLimit",
        security_group_ids=[default_security_group.id],
        system_disk={
            "category": "cloud_ssd",
            "description": "Test For Terraform",
            "name": "terraform-example",
            "size": 40,
            "delete_with_instance": False,
        },
        user_data="xxxxxxx",
        vswitch_id=default_switch.id,
        vpc_id=default_network.id,
        zone_id=default.zones[0].id,
        template_tags={
            "Create": "Terraform",
            "For": "example",
        },
        network_interfaces={
            "name": "eth0",
            "description": "hello1",
            "primary_ip": "10.0.0.2",
            "security_group_id": default_security_group.id,
            "vswitch_id": default_switch.id,
        },
        data_disks=[
            {
                "name": "disk1",
                "description": "description",
                "delete_with_instance": True,
                "category": "cloud",
                "encrypted": False,
                "performance_level": "PL0",
                "size": 20,
            },
            {
                "name": "disk2",
                "description": "description2",
                "delete_with_instance": True,
                "category": "cloud",
                "encrypted": False,
                "performance_level": "PL0",
                "size": 20,
            },
        ])
    example = alicloud.ecs.get_ecs_launch_templates_output(ids=[default_ecs_launch_template.id])
    pulumi.export("firstEcsLaunchTemplateId", example.templates[0].id)
    ```


    :param bool enable_details: Default to `false`. Set it to `true` can output more details about resource attributes.
    :param Sequence[str] ids: A list of Launch Template IDs.
    :param str launch_template_name: The Launch Template Name.
    :param str name_regex: A regex string to filter results by Launch Template name.
    :param str output_file: File name where to save data source results (after running `pulumi preview`).
    :param str template_resource_group_id: The template resource group id.
    :param Mapping[str, str] template_tags: The template tags.
    """
    __args__ = dict()
    __args__['enableDetails'] = enable_details
    __args__['ids'] = ids
    __args__['launchTemplateName'] = launch_template_name
    __args__['nameRegex'] = name_regex
    __args__['outputFile'] = output_file
    __args__['templateResourceGroupId'] = template_resource_group_id
    __args__['templateTags'] = template_tags
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('alicloud:ecs/getEcsLaunchTemplates:getEcsLaunchTemplates', __args__, opts=opts, typ=GetEcsLaunchTemplatesResult).value

    return AwaitableGetEcsLaunchTemplatesResult(
        enable_details=pulumi.get(__ret__, 'enable_details'),
        id=pulumi.get(__ret__, 'id'),
        ids=pulumi.get(__ret__, 'ids'),
        launch_template_name=pulumi.get(__ret__, 'launch_template_name'),
        name_regex=pulumi.get(__ret__, 'name_regex'),
        names=pulumi.get(__ret__, 'names'),
        output_file=pulumi.get(__ret__, 'output_file'),
        template_resource_group_id=pulumi.get(__ret__, 'template_resource_group_id'),
        template_tags=pulumi.get(__ret__, 'template_tags'),
        templates=pulumi.get(__ret__, 'templates'))
def get_ecs_launch_templates_output(enable_details: Optional[pulumi.Input[Optional[bool]]] = None,
                                    ids: Optional[pulumi.Input[Optional[Sequence[str]]]] = None,
                                    launch_template_name: Optional[pulumi.Input[Optional[str]]] = None,
                                    name_regex: Optional[pulumi.Input[Optional[str]]] = None,
                                    output_file: Optional[pulumi.Input[Optional[str]]] = None,
                                    template_resource_group_id: Optional[pulumi.Input[Optional[str]]] = None,
                                    template_tags: Optional[pulumi.Input[Optional[Mapping[str, str]]]] = None,
                                    opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetEcsLaunchTemplatesResult]:
    """
    This data source provides the Ecs Launch Templates of the current Alibaba Cloud user.

    > **NOTE:** Available since v1.120.0.

    ## Example Usage

    Basic Usage

    ```python
    import pulumi
    import pulumi_alicloud as alicloud

    default = alicloud.get_zones(available_disk_category="cloud_efficiency",
        available_resource_creation="VSwitch")
    default_get_instance_types = alicloud.ecs.get_instance_types(availability_zone=default.zones[0].id)
    default_get_images = alicloud.ecs.get_images(name_regex="^ubuntu_18.*64",
        owners="system")
    default_network = alicloud.vpc.Network("default",
        vpc_name="terraform-example",
        cidr_block="172.17.3.0/24")
    default_switch = alicloud.vpc.Switch("default",
        vswitch_name="terraform-example",
        cidr_block="172.17.3.0/24",
        vpc_id=default_network.id,
        zone_id=default.zones[0].id)
    default_security_group = alicloud.ecs.SecurityGroup("default",
        name="terraform-example",
        vpc_id=default_network.id)
    default_ecs_launch_template = alicloud.ecs.EcsLaunchTemplate("default",
        launch_template_name="terraform-example",
        description="terraform-example",
        image_id=default_get_images.images[0].id,
        host_name="terraform-example",
        instance_charge_type="PrePaid",
        instance_name="terraform-example",
        instance_type=default_get_instance_types.instance_types[0].id,
        internet_charge_type="PayByBandwidth",
        internet_max_bandwidth_in=5,
        internet_max_bandwidth_out=5,
        io_optimized="optimized",
        key_pair_name="key_pair_name",
        ram_role_name="ram_role_name",
        network_type="vpc",
        security_enhancement_strategy="Active",
        spot_price_limit=5,
        spot_strategy="SpotWithPriceLimit",
        security_group_ids=[default_security_group.id],
        system_disk={
            "category": "cloud_ssd",
            "description": "Test For Terraform",
            "name": "terraform-example",
            "size": 40,
            "delete_with_instance": False,
        },
        user_data="xxxxxxx",
        vswitch_id=default_switch.id,
        vpc_id=default_network.id,
        zone_id=default.zones[0].id,
        template_tags={
            "Create": "Terraform",
            "For": "example",
        },
        network_interfaces={
            "name": "eth0",
            "description": "hello1",
            "primary_ip": "10.0.0.2",
            "security_group_id": default_security_group.id,
            "vswitch_id": default_switch.id,
        },
        data_disks=[
            {
                "name": "disk1",
                "description": "description",
                "delete_with_instance": True,
                "category": "cloud",
                "encrypted": False,
                "performance_level": "PL0",
                "size": 20,
            },
            {
                "name": "disk2",
                "description": "description2",
                "delete_with_instance": True,
                "category": "cloud",
                "encrypted": False,
                "performance_level": "PL0",
                "size": 20,
            },
        ])
    example = alicloud.ecs.get_ecs_launch_templates_output(ids=[default_ecs_launch_template.id])
    pulumi.export("firstEcsLaunchTemplateId", example.templates[0].id)
    ```


    :param bool enable_details: Default to `false`. Set it to `true` can output more details about resource attributes.
    :param Sequence[str] ids: A list of Launch Template IDs.
    :param str launch_template_name: The Launch Template Name.
    :param str name_regex: A regex string to filter results by Launch Template name.
    :param str output_file: File name where to save data source results (after running `pulumi preview`).
    :param str template_resource_group_id: The template resource group id.
    :param Mapping[str, str] template_tags: The template tags.
    """
    __args__ = dict()
    __args__['enableDetails'] = enable_details
    __args__['ids'] = ids
    __args__['launchTemplateName'] = launch_template_name
    __args__['nameRegex'] = name_regex
    __args__['outputFile'] = output_file
    __args__['templateResourceGroupId'] = template_resource_group_id
    __args__['templateTags'] = template_tags
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('alicloud:ecs/getEcsLaunchTemplates:getEcsLaunchTemplates', __args__, opts=opts, typ=GetEcsLaunchTemplatesResult)
    return __ret__.apply(lambda __response__: GetEcsLaunchTemplatesResult(
        enable_details=pulumi.get(__response__, 'enable_details'),
        id=pulumi.get(__response__, 'id'),
        ids=pulumi.get(__response__, 'ids'),
        launch_template_name=pulumi.get(__response__, 'launch_template_name'),
        name_regex=pulumi.get(__response__, 'name_regex'),
        names=pulumi.get(__response__, 'names'),
        output_file=pulumi.get(__response__, 'output_file'),
        template_resource_group_id=pulumi.get(__response__, 'template_resource_group_id'),
        template_tags=pulumi.get(__response__, 'template_tags'),
        templates=pulumi.get(__response__, 'templates')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs

__all__ = [
    'GetServiceHybridDoubleWritesResult',
    'AwaitableGetServiceHybridDoubleWritesResult',
    'get_service_hybrid_double_writes',
    'get_service_hybrid_double_writes_output',
]

@pulumi.output_type
class GetServiceHybridDoubleWritesResult:
    """
    A collection of values returned by getServiceHybridDoubleWrites.
    """
    def __init__(__self__, hybrid_double_writes=None, id=None, ids=None, namespace=None, output_file=None, source_namespace=None, source_user_id=None, user_id=None):
        if hybrid_double_writes and not isinstance(hybrid_double_writes, list):
            raise TypeError("Expected argument 'hybrid_double_writes' to be a list")
        pulumi.set(__self__, "hybrid_double_writes", hybrid_double_writes)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ids and not isinstance(ids, list):
            raise TypeError("Expected argument 'ids' to be a list")
        pulumi.set(__self__, "ids", ids)
        if namespace and not isinstance(namespace, str):
            raise TypeError("Expected argument 'namespace' to be a str")
        pulumi.set(__self__, "namespace", namespace)
        if output_file and not isinstance(output_file, str):
            raise TypeError("Expected argument 'output_file' to be a str")
        pulumi.set(__self__, "output_file", output_file)
        if source_namespace and not isinstance(source_namespace, str):
            raise TypeError("Expected argument 'source_namespace' to be a str")
        pulumi.set(__self__, "source_namespace", source_namespace)
        if source_user_id and not isinstance(source_user_id, str):
            raise TypeError("Expected argument 'source_user_id' to be a str")
        pulumi.set(__self__, "source_user_id", source_user_id)
        if user_id and not isinstance(user_id, str):
            raise TypeError("Expected argument 'user_id' to be a str")
        pulumi.set(__self__, "user_id", user_id)

    @property
    @pulumi.getter(name="hybridDoubleWrites")
    def hybrid_double_writes(self) -> Sequence['outputs.GetServiceHybridDoubleWritesHybridDoubleWriteResult']:
        """
        A list of Hybrid Double Writes. Each element contains the following attributes:
        """
        return pulumi.get(self, "hybrid_double_writes")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def ids(self) -> Sequence[str]:
        return pulumi.get(self, "ids")

    @property
    @pulumi.getter
    def namespace(self) -> Optional[str]:
        """
        Target Namespace.
        """
        return pulumi.get(self, "namespace")

    @property
    @pulumi.getter(name="outputFile")
    def output_file(self) -> Optional[str]:
        return pulumi.get(self, "output_file")

    @property
    @pulumi.getter(name="sourceNamespace")
    def source_namespace(self) -> Optional[str]:
        """
        Source Namespace.
        """
        return pulumi.get(self, "source_namespace")

    @property
    @pulumi.getter(name="sourceUserId")
    def source_user_id(self) -> Optional[str]:
        """
        Source UserId.
        """
        return pulumi.get(self, "source_user_id")

    @property
    @pulumi.getter(name="userId")
    def user_id(self) -> Optional[str]:
        """
        Target UserId.
        """
        return pulumi.get(self, "user_id")


class AwaitableGetServiceHybridDoubleWritesResult(GetServiceHybridDoubleWritesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetServiceHybridDoubleWritesResult(
            hybrid_double_writes=self.hybrid_double_writes,
            id=self.id,
            ids=self.ids,
            namespace=self.namespace,
            output_file=self.output_file,
            source_namespace=self.source_namespace,
            source_user_id=self.source_user_id,
            user_id=self.user_id)


def get_service_hybrid_double_writes(ids: Optional[Sequence[str]] = None,
                                     namespace: Optional[str] = None,
                                     output_file: Optional[str] = None,
                                     source_namespace: Optional[str] = None,
                                     source_user_id: Optional[str] = None,
                                     user_id: Optional[str] = None,
                                     opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetServiceHybridDoubleWritesResult:
    """
    This data source provides the Cloud Monitor Service Hybrid Double Writes of the current Alibaba Cloud user.

    > **NOTE:** Available since v1.220.0.

    ## Example Usage

    Basic Usage

    ```python
    import pulumi
    import pulumi_alicloud as alicloud

    default = alicloud.get_account()
    source = alicloud.cms.Namespace("source", namespace="your-source-namespace")
    default_namespace = alicloud.cms.Namespace("default", namespace="your-namespace")
    default_service_hybrid_double_write = alicloud.cloudmonitor.ServiceHybridDoubleWrite("default",
        source_namespace=source.id,
        source_user_id=default.id,
        namespace=default_namespace.id,
        user_id=default.id)
    ids = alicloud.cloudmonitor.get_service_hybrid_double_writes_output(ids=[default_service_hybrid_double_write.id])
    pulumi.export("cloudMonitorServiceHybridDoubleWritesId1", ids.hybrid_double_writes[0].id)
    ```


    :param Sequence[str] ids: A list of Hybrid Double Write IDs.
    :param str namespace: Target Namespace.
    :param str output_file: File name where to save data source results (after running `pulumi preview`).
    :param str source_namespace: Source Namespace.
    :param str source_user_id: Source UserId.
    :param str user_id: Target UserId.
    """
    __args__ = dict()
    __args__['ids'] = ids
    __args__['namespace'] = namespace
    __args__['outputFile'] = output_file
    __args__['sourceNamespace'] = source_namespace
    __args__['sourceUserId'] = source_user_id
    __args__['userId'] = user_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('alicloud:cloudmonitor/getServiceHybridDoubleWrites:getServiceHybridDoubleWrites', __args__, opts=opts, typ=GetServiceHybridDoubleWritesResult).value

    return AwaitableGetServiceHybridDoubleWritesResult(
        hybrid_double_writes=pulumi.get(__ret__, 'hybrid_double_writes'),
        id=pulumi.get(__ret__, 'id'),
        ids=pulumi.get(__ret__, 'ids'),
        namespace=pulumi.get(__ret__, 'namespace'),
        output_file=pulumi.get(__ret__, 'output_file'),
        source_namespace=pulumi.get(__ret__, 'source_namespace'),
        source_user_id=pulumi.get(__ret__, 'source_user_id'),
        user_id=pulumi.get(__ret__, 'user_id'))
def get_service_hybrid_double_writes_output(ids: Optional[pulumi.Input[Optional[Sequence[str]]]] = None,
                                            namespace: Optional[pulumi.Input[Optional[str]]] = None,
                                            output_file: Optional[pulumi.Input[Optional[str]]] = None,
                                            source_namespace: Optional[pulumi.Input[Optional[str]]] = None,
                                            source_user_id: Optional[pulumi.Input[Optional[str]]] = None,
                                            user_id: Optional[pulumi.Input[Optional[str]]] = None,
                                            opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetServiceHybridDoubleWritesResult]:
    """
    This data source provides the Cloud Monitor Service Hybrid Double Writes of the current Alibaba Cloud user.

    > **NOTE:** Available since v1.220.0.

    ## Example Usage

    Basic Usage

    ```python
    import pulumi
    import pulumi_alicloud as alicloud

    default = alicloud.get_account()
    source = alicloud.cms.Namespace("source", namespace="your-source-namespace")
    default_namespace = alicloud.cms.Namespace("default", namespace="your-namespace")
    default_service_hybrid_double_write = alicloud.cloudmonitor.ServiceHybridDoubleWrite("default",
        source_namespace=source.id,
        source_user_id=default.id,
        namespace=default_namespace.id,
        user_id=default.id)
    ids = alicloud.cloudmonitor.get_service_hybrid_double_writes_output(ids=[default_service_hybrid_double_write.id])
    pulumi.export("cloudMonitorServiceHybridDoubleWritesId1", ids.hybrid_double_writes[0].id)
    ```


    :param Sequence[str] ids: A list of Hybrid Double Write IDs.
    :param str namespace: Target Namespace.
    :param str output_file: File name where to save data source results (after running `pulumi preview`).
    :param str source_namespace: Source Namespace.
    :param str source_user_id: Source UserId.
    :param str user_id: Target UserId.
    """
    __args__ = dict()
    __args__['ids'] = ids
    __args__['namespace'] = namespace
    __args__['outputFile'] = output_file
    __args__['sourceNamespace'] = source_namespace
    __args__['sourceUserId'] = source_user_id
    __args__['userId'] = user_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('alicloud:cloudmonitor/getServiceHybridDoubleWrites:getServiceHybridDoubleWrites', __args__, opts=opts, typ=GetServiceHybridDoubleWritesResult)
    return __ret__.apply(lambda __response__: GetServiceHybridDoubleWritesResult(
        hybrid_double_writes=pulumi.get(__response__, 'hybrid_double_writes'),
        id=pulumi.get(__response__, 'id'),
        ids=pulumi.get(__response__, 'ids'),
        namespace=pulumi.get(__response__, 'namespace'),
        output_file=pulumi.get(__response__, 'output_file'),
        source_namespace=pulumi.get(__response__, 'source_namespace'),
        source_user_id=pulumi.get(__response__, 'source_user_id'),
        user_id=pulumi.get(__response__, 'user_id')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['BindingArgs', 'Binding']

@pulumi.input_type
class BindingArgs:
    def __init__(__self__, *,
                 binding_key: pulumi.Input[str],
                 binding_type: pulumi.Input[str],
                 destination_name: pulumi.Input[str],
                 instance_id: pulumi.Input[str],
                 source_exchange: pulumi.Input[str],
                 virtual_host_name: pulumi.Input[str],
                 argument: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a Binding resource.
        :param pulumi.Input[str] binding_key: The Binding Key.
               * For a non-topic source exchange: The binding key can contain only letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
               The binding key must be 1 to 255 characters in length.
               * For a topic source exchange: The binding key can contain letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
               If the binding key contains a number sign (#), the binding key must start with a number sign (#) followed by a period (.) or end with a number sign (#) that follows a period (.).
               The binding key must be 1 to 255 characters in length.
        :param pulumi.Input[str] binding_type: The type of the object that you want to bind to the source exchange. Valid values: `EXCHANGE`, `QUEUE`.
        :param pulumi.Input[str] destination_name: The name of the object that you want to bind to the source exchange.
        :param pulumi.Input[str] instance_id: The ID of the instance.
        :param pulumi.Input[str] source_exchange: The name of the source exchange.
        :param pulumi.Input[str] virtual_host_name: The name of the vhost.
        :param pulumi.Input[str] argument: The key-value pairs that are configured for the headers attributes of a message. Default value: `x-match:all`. Valid values:
               - `x-match:all`: A headers exchange routes a message to a queue only if all binding attributes of the queue except for x-match match the headers attributes of the message.
               - `x-match:any`: A headers exchange routes a message to a queue if one or more binding attributes of the queue except for x-match match the headers attributes of the message.
               - > **NOTE:** If the exchange type is not 'HEADERS', the `argument` should not been set, otherwise, there are always "forces replacement" changes.
        """
        pulumi.set(__self__, "binding_key", binding_key)
        pulumi.set(__self__, "binding_type", binding_type)
        pulumi.set(__self__, "destination_name", destination_name)
        pulumi.set(__self__, "instance_id", instance_id)
        pulumi.set(__self__, "source_exchange", source_exchange)
        pulumi.set(__self__, "virtual_host_name", virtual_host_name)
        if argument is not None:
            pulumi.set(__self__, "argument", argument)

    @property
    @pulumi.getter(name="bindingKey")
    def binding_key(self) -> pulumi.Input[str]:
        """
        The Binding Key.
        * For a non-topic source exchange: The binding key can contain only letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
        The binding key must be 1 to 255 characters in length.
        * For a topic source exchange: The binding key can contain letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
        If the binding key contains a number sign (#), the binding key must start with a number sign (#) followed by a period (.) or end with a number sign (#) that follows a period (.).
        The binding key must be 1 to 255 characters in length.
        """
        return pulumi.get(self, "binding_key")

    @binding_key.setter
    def binding_key(self, value: pulumi.Input[str]):
        pulumi.set(self, "binding_key", value)

    @property
    @pulumi.getter(name="bindingType")
    def binding_type(self) -> pulumi.Input[str]:
        """
        The type of the object that you want to bind to the source exchange. Valid values: `EXCHANGE`, `QUEUE`.
        """
        return pulumi.get(self, "binding_type")

    @binding_type.setter
    def binding_type(self, value: pulumi.Input[str]):
        pulumi.set(self, "binding_type", value)

    @property
    @pulumi.getter(name="destinationName")
    def destination_name(self) -> pulumi.Input[str]:
        """
        The name of the object that you want to bind to the source exchange.
        """
        return pulumi.get(self, "destination_name")

    @destination_name.setter
    def destination_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "destination_name", value)

    @property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> pulumi.Input[str]:
        """
        The ID of the instance.
        """
        return pulumi.get(self, "instance_id")

    @instance_id.setter
    def instance_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "instance_id", value)

    @property
    @pulumi.getter(name="sourceExchange")
    def source_exchange(self) -> pulumi.Input[str]:
        """
        The name of the source exchange.
        """
        return pulumi.get(self, "source_exchange")

    @source_exchange.setter
    def source_exchange(self, value: pulumi.Input[str]):
        pulumi.set(self, "source_exchange", value)

    @property
    @pulumi.getter(name="virtualHostName")
    def virtual_host_name(self) -> pulumi.Input[str]:
        """
        The name of the vhost.
        """
        return pulumi.get(self, "virtual_host_name")

    @virtual_host_name.setter
    def virtual_host_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "virtual_host_name", value)

    @property
    @pulumi.getter
    def argument(self) -> Optional[pulumi.Input[str]]:
        """
        The key-value pairs that are configured for the headers attributes of a message. Default value: `x-match:all`. Valid values:
        - `x-match:all`: A headers exchange routes a message to a queue only if all binding attributes of the queue except for x-match match the headers attributes of the message.
        - `x-match:any`: A headers exchange routes a message to a queue if one or more binding attributes of the queue except for x-match match the headers attributes of the message.
        - > **NOTE:** If the exchange type is not 'HEADERS', the `argument` should not been set, otherwise, there are always "forces replacement" changes.
        """
        return pulumi.get(self, "argument")

    @argument.setter
    def argument(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "argument", value)


@pulumi.input_type
class _BindingState:
    def __init__(__self__, *,
                 argument: Optional[pulumi.Input[str]] = None,
                 binding_key: Optional[pulumi.Input[str]] = None,
                 binding_type: Optional[pulumi.Input[str]] = None,
                 destination_name: Optional[pulumi.Input[str]] = None,
                 instance_id: Optional[pulumi.Input[str]] = None,
                 source_exchange: Optional[pulumi.Input[str]] = None,
                 virtual_host_name: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Binding resources.
        :param pulumi.Input[str] argument: The key-value pairs that are configured for the headers attributes of a message. Default value: `x-match:all`. Valid values:
               - `x-match:all`: A headers exchange routes a message to a queue only if all binding attributes of the queue except for x-match match the headers attributes of the message.
               - `x-match:any`: A headers exchange routes a message to a queue if one or more binding attributes of the queue except for x-match match the headers attributes of the message.
               - > **NOTE:** If the exchange type is not 'HEADERS', the `argument` should not been set, otherwise, there are always "forces replacement" changes.
        :param pulumi.Input[str] binding_key: The Binding Key.
               * For a non-topic source exchange: The binding key can contain only letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
               The binding key must be 1 to 255 characters in length.
               * For a topic source exchange: The binding key can contain letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
               If the binding key contains a number sign (#), the binding key must start with a number sign (#) followed by a period (.) or end with a number sign (#) that follows a period (.).
               The binding key must be 1 to 255 characters in length.
        :param pulumi.Input[str] binding_type: The type of the object that you want to bind to the source exchange. Valid values: `EXCHANGE`, `QUEUE`.
        :param pulumi.Input[str] destination_name: The name of the object that you want to bind to the source exchange.
        :param pulumi.Input[str] instance_id: The ID of the instance.
        :param pulumi.Input[str] source_exchange: The name of the source exchange.
        :param pulumi.Input[str] virtual_host_name: The name of the vhost.
        """
        if argument is not None:
            pulumi.set(__self__, "argument", argument)
        if binding_key is not None:
            pulumi.set(__self__, "binding_key", binding_key)
        if binding_type is not None:
            pulumi.set(__self__, "binding_type", binding_type)
        if destination_name is not None:
            pulumi.set(__self__, "destination_name", destination_name)
        if instance_id is not None:
            pulumi.set(__self__, "instance_id", instance_id)
        if source_exchange is not None:
            pulumi.set(__self__, "source_exchange", source_exchange)
        if virtual_host_name is not None:
            pulumi.set(__self__, "virtual_host_name", virtual_host_name)

    @property
    @pulumi.getter
    def argument(self) -> Optional[pulumi.Input[str]]:
        """
        The key-value pairs that are configured for the headers attributes of a message. Default value: `x-match:all`. Valid values:
        - `x-match:all`: A headers exchange routes a message to a queue only if all binding attributes of the queue except for x-match match the headers attributes of the message.
        - `x-match:any`: A headers exchange routes a message to a queue if one or more binding attributes of the queue except for x-match match the headers attributes of the message.
        - > **NOTE:** If the exchange type is not 'HEADERS', the `argument` should not been set, otherwise, there are always "forces replacement" changes.
        """
        return pulumi.get(self, "argument")

    @argument.setter
    def argument(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "argument", value)

    @property
    @pulumi.getter(name="bindingKey")
    def binding_key(self) -> Optional[pulumi.Input[str]]:
        """
        The Binding Key.
        * For a non-topic source exchange: The binding key can contain only letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
        The binding key must be 1 to 255 characters in length.
        * For a topic source exchange: The binding key can contain letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
        If the binding key contains a number sign (#), the binding key must start with a number sign (#) followed by a period (.) or end with a number sign (#) that follows a period (.).
        The binding key must be 1 to 255 characters in length.
        """
        return pulumi.get(self, "binding_key")

    @binding_key.setter
    def binding_key(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "binding_key", value)

    @property
    @pulumi.getter(name="bindingType")
    def binding_type(self) -> Optional[pulumi.Input[str]]:
        """
        The type of the object that you want to bind to the source exchange. Valid values: `EXCHANGE`, `QUEUE`.
        """
        return pulumi.get(self, "binding_type")

    @binding_type.setter
    def binding_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "binding_type", value)

    @property
    @pulumi.getter(name="destinationName")
    def destination_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the object that you want to bind to the source exchange.
        """
        return pulumi.get(self, "destination_name")

    @destination_name.setter
    def destination_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "destination_name", value)

    @property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the instance.
        """
        return pulumi.get(self, "instance_id")

    @instance_id.setter
    def instance_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "instance_id", value)

    @property
    @pulumi.getter(name="sourceExchange")
    def source_exchange(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the source exchange.
        """
        return pulumi.get(self, "source_exchange")

    @source_exchange.setter
    def source_exchange(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "source_exchange", value)

    @property
    @pulumi.getter(name="virtualHostName")
    def virtual_host_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the vhost.
        """
        return pulumi.get(self, "virtual_host_name")

    @virtual_host_name.setter
    def virtual_host_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "virtual_host_name", value)


class Binding(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 argument: Optional[pulumi.Input[str]] = None,
                 binding_key: Optional[pulumi.Input[str]] = None,
                 binding_type: Optional[pulumi.Input[str]] = None,
                 destination_name: Optional[pulumi.Input[str]] = None,
                 instance_id: Optional[pulumi.Input[str]] = None,
                 source_exchange: Optional[pulumi.Input[str]] = None,
                 virtual_host_name: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Provides a RabbitMQ (AMQP) Binding resource.

        For information about RabbitMQ (AMQP) Binding and how to use it, see [What is Binding](https://www.alibabacloud.com/help/en/message-queue-for-rabbitmq/latest/createbinding).

        > **NOTE:** Available since v1.135.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        default = alicloud.amqp.Instance("default",
            instance_type="enterprise",
            max_tps="3000",
            queue_capacity="200",
            storage_size="700",
            support_eip=False,
            max_eip_tps="128",
            payment_type="Subscription",
            period=1)
        default_virtual_host = alicloud.amqp.VirtualHost("default",
            instance_id=default.id,
            virtual_host_name="tf-example")
        default_exchange = alicloud.amqp.Exchange("default",
            auto_delete_state=False,
            exchange_name="tf-example",
            exchange_type="HEADERS",
            instance_id=default.id,
            internal=False,
            virtual_host_name=default_virtual_host.virtual_host_name)
        default_queue = alicloud.amqp.Queue("default",
            instance_id=default.id,
            queue_name="tf-example",
            virtual_host_name=default_virtual_host.virtual_host_name)
        default_binding = alicloud.amqp.Binding("default",
            argument="x-match:all",
            binding_key=default_queue.queue_name,
            binding_type="QUEUE",
            destination_name="tf-example",
            instance_id=default.id,
            source_exchange=default_exchange.exchange_name,
            virtual_host_name=default_virtual_host.virtual_host_name)
        ```

        ## Import

        RabbitMQ (AMQP) Binding can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:amqp/binding:Binding example <instance_id>:<virtual_host_name>:<source_exchange>:<destination_name>
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] argument: The key-value pairs that are configured for the headers attributes of a message. Default value: `x-match:all`. Valid values:
               - `x-match:all`: A headers exchange routes a message to a queue only if all binding attributes of the queue except for x-match match the headers attributes of the message.
               - `x-match:any`: A headers exchange routes a message to a queue if one or more binding attributes of the queue except for x-match match the headers attributes of the message.
               - > **NOTE:** If the exchange type is not 'HEADERS', the `argument` should not been set, otherwise, there are always "forces replacement" changes.
        :param pulumi.Input[str] binding_key: The Binding Key.
               * For a non-topic source exchange: The binding key can contain only letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
               The binding key must be 1 to 255 characters in length.
               * For a topic source exchange: The binding key can contain letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
               If the binding key contains a number sign (#), the binding key must start with a number sign (#) followed by a period (.) or end with a number sign (#) that follows a period (.).
               The binding key must be 1 to 255 characters in length.
        :param pulumi.Input[str] binding_type: The type of the object that you want to bind to the source exchange. Valid values: `EXCHANGE`, `QUEUE`.
        :param pulumi.Input[str] destination_name: The name of the object that you want to bind to the source exchange.
        :param pulumi.Input[str] instance_id: The ID of the instance.
        :param pulumi.Input[str] source_exchange: The name of the source exchange.
        :param pulumi.Input[str] virtual_host_name: The name of the vhost.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: BindingArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a RabbitMQ (AMQP) Binding resource.

        For information about RabbitMQ (AMQP) Binding and how to use it, see [What is Binding](https://www.alibabacloud.com/help/en/message-queue-for-rabbitmq/latest/createbinding).

        > **NOTE:** Available since v1.135.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        default = alicloud.amqp.Instance("default",
            instance_type="enterprise",
            max_tps="3000",
            queue_capacity="200",
            storage_size="700",
            support_eip=False,
            max_eip_tps="128",
            payment_type="Subscription",
            period=1)
        default_virtual_host = alicloud.amqp.VirtualHost("default",
            instance_id=default.id,
            virtual_host_name="tf-example")
        default_exchange = alicloud.amqp.Exchange("default",
            auto_delete_state=False,
            exchange_name="tf-example",
            exchange_type="HEADERS",
            instance_id=default.id,
            internal=False,
            virtual_host_name=default_virtual_host.virtual_host_name)
        default_queue = alicloud.amqp.Queue("default",
            instance_id=default.id,
            queue_name="tf-example",
            virtual_host_name=default_virtual_host.virtual_host_name)
        default_binding = alicloud.amqp.Binding("default",
            argument="x-match:all",
            binding_key=default_queue.queue_name,
            binding_type="QUEUE",
            destination_name="tf-example",
            instance_id=default.id,
            source_exchange=default_exchange.exchange_name,
            virtual_host_name=default_virtual_host.virtual_host_name)
        ```

        ## Import

        RabbitMQ (AMQP) Binding can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:amqp/binding:Binding example <instance_id>:<virtual_host_name>:<source_exchange>:<destination_name>
        ```

        :param str resource_name: The name of the resource.
        :param BindingArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(BindingArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 argument: Optional[pulumi.Input[str]] = None,
                 binding_key: Optional[pulumi.Input[str]] = None,
                 binding_type: Optional[pulumi.Input[str]] = None,
                 destination_name: Optional[pulumi.Input[str]] = None,
                 instance_id: Optional[pulumi.Input[str]] = None,
                 source_exchange: Optional[pulumi.Input[str]] = None,
                 virtual_host_name: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = BindingArgs.__new__(BindingArgs)

            __props__.__dict__["argument"] = argument
            if binding_key is None and not opts.urn:
                raise TypeError("Missing required property 'binding_key'")
            __props__.__dict__["binding_key"] = binding_key
            if binding_type is None and not opts.urn:
                raise TypeError("Missing required property 'binding_type'")
            __props__.__dict__["binding_type"] = binding_type
            if destination_name is None and not opts.urn:
                raise TypeError("Missing required property 'destination_name'")
            __props__.__dict__["destination_name"] = destination_name
            if instance_id is None and not opts.urn:
                raise TypeError("Missing required property 'instance_id'")
            __props__.__dict__["instance_id"] = instance_id
            if source_exchange is None and not opts.urn:
                raise TypeError("Missing required property 'source_exchange'")
            __props__.__dict__["source_exchange"] = source_exchange
            if virtual_host_name is None and not opts.urn:
                raise TypeError("Missing required property 'virtual_host_name'")
            __props__.__dict__["virtual_host_name"] = virtual_host_name
        super(Binding, __self__).__init__(
            'alicloud:amqp/binding:Binding',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            argument: Optional[pulumi.Input[str]] = None,
            binding_key: Optional[pulumi.Input[str]] = None,
            binding_type: Optional[pulumi.Input[str]] = None,
            destination_name: Optional[pulumi.Input[str]] = None,
            instance_id: Optional[pulumi.Input[str]] = None,
            source_exchange: Optional[pulumi.Input[str]] = None,
            virtual_host_name: Optional[pulumi.Input[str]] = None) -> 'Binding':
        """
        Get an existing Binding resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] argument: The key-value pairs that are configured for the headers attributes of a message. Default value: `x-match:all`. Valid values:
               - `x-match:all`: A headers exchange routes a message to a queue only if all binding attributes of the queue except for x-match match the headers attributes of the message.
               - `x-match:any`: A headers exchange routes a message to a queue if one or more binding attributes of the queue except for x-match match the headers attributes of the message.
               - > **NOTE:** If the exchange type is not 'HEADERS', the `argument` should not been set, otherwise, there are always "forces replacement" changes.
        :param pulumi.Input[str] binding_key: The Binding Key.
               * For a non-topic source exchange: The binding key can contain only letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
               The binding key must be 1 to 255 characters in length.
               * For a topic source exchange: The binding key can contain letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
               If the binding key contains a number sign (#), the binding key must start with a number sign (#) followed by a period (.) or end with a number sign (#) that follows a period (.).
               The binding key must be 1 to 255 characters in length.
        :param pulumi.Input[str] binding_type: The type of the object that you want to bind to the source exchange. Valid values: `EXCHANGE`, `QUEUE`.
        :param pulumi.Input[str] destination_name: The name of the object that you want to bind to the source exchange.
        :param pulumi.Input[str] instance_id: The ID of the instance.
        :param pulumi.Input[str] source_exchange: The name of the source exchange.
        :param pulumi.Input[str] virtual_host_name: The name of the vhost.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _BindingState.__new__(_BindingState)

        __props__.__dict__["argument"] = argument
        __props__.__dict__["binding_key"] = binding_key
        __props__.__dict__["binding_type"] = binding_type
        __props__.__dict__["destination_name"] = destination_name
        __props__.__dict__["instance_id"] = instance_id
        __props__.__dict__["source_exchange"] = source_exchange
        __props__.__dict__["virtual_host_name"] = virtual_host_name
        return Binding(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def argument(self) -> pulumi.Output[str]:
        """
        The key-value pairs that are configured for the headers attributes of a message. Default value: `x-match:all`. Valid values:
        - `x-match:all`: A headers exchange routes a message to a queue only if all binding attributes of the queue except for x-match match the headers attributes of the message.
        - `x-match:any`: A headers exchange routes a message to a queue if one or more binding attributes of the queue except for x-match match the headers attributes of the message.
        - > **NOTE:** If the exchange type is not 'HEADERS', the `argument` should not been set, otherwise, there are always "forces replacement" changes.
        """
        return pulumi.get(self, "argument")

    @property
    @pulumi.getter(name="bindingKey")
    def binding_key(self) -> pulumi.Output[str]:
        """
        The Binding Key.
        * For a non-topic source exchange: The binding key can contain only letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
        The binding key must be 1 to 255 characters in length.
        * For a topic source exchange: The binding key can contain letters, digits, hyphens (-), underscores (_), periods (.), and at signs (@).
        If the binding key contains a number sign (#), the binding key must start with a number sign (#) followed by a period (.) or end with a number sign (#) that follows a period (.).
        The binding key must be 1 to 255 characters in length.
        """
        return pulumi.get(self, "binding_key")

    @property
    @pulumi.getter(name="bindingType")
    def binding_type(self) -> pulumi.Output[str]:
        """
        The type of the object that you want to bind to the source exchange. Valid values: `EXCHANGE`, `QUEUE`.
        """
        return pulumi.get(self, "binding_type")

    @property
    @pulumi.getter(name="destinationName")
    def destination_name(self) -> pulumi.Output[str]:
        """
        The name of the object that you want to bind to the source exchange.
        """
        return pulumi.get(self, "destination_name")

    @property
    @pulumi.getter(name="instanceId")
    def instance_id(self) -> pulumi.Output[str]:
        """
        The ID of the instance.
        """
        return pulumi.get(self, "instance_id")

    @property
    @pulumi.getter(name="sourceExchange")
    def source_exchange(self) -> pulumi.Output[str]:
        """
        The name of the source exchange.
        """
        return pulumi.get(self, "source_exchange")

    @property
    @pulumi.getter(name="virtualHostName")
    def virtual_host_name(self) -> pulumi.Output[str]:
        """
        The name of the vhost.
        """
        return pulumi.get(self, "virtual_host_name")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['ApplicationArgs', 'Application']

@pulumi.input_type
class ApplicationArgs:
    def __init__(__self__, *,
                 cluster_name: pulumi.Input[str],
                 template: pulumi.Input[str],
                 blue_green: Optional[pulumi.Input[bool]] = None,
                 blue_green_confirm: Optional[pulumi.Input[bool]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 environment: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 latest_image: Optional[pulumi.Input[bool]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 version: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a Application resource.
        """
        pulumi.set(__self__, "cluster_name", cluster_name)
        pulumi.set(__self__, "template", template)
        if blue_green is not None:
            pulumi.set(__self__, "blue_green", blue_green)
        if blue_green_confirm is not None:
            pulumi.set(__self__, "blue_green_confirm", blue_green_confirm)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if environment is not None:
            pulumi.set(__self__, "environment", environment)
        if latest_image is not None:
            pulumi.set(__self__, "latest_image", latest_image)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if version is not None:
            pulumi.set(__self__, "version", version)

    @property
    @pulumi.getter(name="clusterName")
    def cluster_name(self) -> pulumi.Input[str]:
        return pulumi.get(self, "cluster_name")

    @cluster_name.setter
    def cluster_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "cluster_name", value)

    @property
    @pulumi.getter
    def template(self) -> pulumi.Input[str]:
        return pulumi.get(self, "template")

    @template.setter
    def template(self, value: pulumi.Input[str]):
        pulumi.set(self, "template", value)

    @property
    @pulumi.getter(name="blueGreen")
    def blue_green(self) -> Optional[pulumi.Input[bool]]:
        return pulumi.get(self, "blue_green")

    @blue_green.setter
    def blue_green(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "blue_green", value)

    @property
    @pulumi.getter(name="blueGreenConfirm")
    def blue_green_confirm(self) -> Optional[pulumi.Input[bool]]:
        return pulumi.get(self, "blue_green_confirm")

    @blue_green_confirm.setter
    def blue_green_confirm(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "blue_green_confirm", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def environment(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        return pulumi.get(self, "environment")

    @environment.setter
    def environment(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "environment", value)

    @property
    @pulumi.getter(name="latestImage")
    def latest_image(self) -> Optional[pulumi.Input[bool]]:
        return pulumi.get(self, "latest_image")

    @latest_image.setter
    def latest_image(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "latest_image", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def version(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "version")

    @version.setter
    def version(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "version", value)


@pulumi.input_type
class _ApplicationState:
    def __init__(__self__, *,
                 blue_green: Optional[pulumi.Input[bool]] = None,
                 blue_green_confirm: Optional[pulumi.Input[bool]] = None,
                 cluster_name: Optional[pulumi.Input[str]] = None,
                 default_domain: Optional[pulumi.Input[str]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 environment: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 latest_image: Optional[pulumi.Input[bool]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 services: Optional[pulumi.Input[Sequence[pulumi.Input['ApplicationServiceArgs']]]] = None,
                 template: Optional[pulumi.Input[str]] = None,
                 version: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Application resources.
        """
        if blue_green is not None:
            pulumi.set(__self__, "blue_green", blue_green)
        if blue_green_confirm is not None:
            pulumi.set(__self__, "blue_green_confirm", blue_green_confirm)
        if cluster_name is not None:
            pulumi.set(__self__, "cluster_name", cluster_name)
        if default_domain is not None:
            pulumi.set(__self__, "default_domain", default_domain)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if environment is not None:
            pulumi.set(__self__, "environment", environment)
        if latest_image is not None:
            pulumi.set(__self__, "latest_image", latest_image)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if services is not None:
            pulumi.set(__self__, "services", services)
        if template is not None:
            pulumi.set(__self__, "template", template)
        if version is not None:
            pulumi.set(__self__, "version", version)

    @property
    @pulumi.getter(name="blueGreen")
    def blue_green(self) -> Optional[pulumi.Input[bool]]:
        return pulumi.get(self, "blue_green")

    @blue_green.setter
    def blue_green(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "blue_green", value)

    @property
    @pulumi.getter(name="blueGreenConfirm")
    def blue_green_confirm(self) -> Optional[pulumi.Input[bool]]:
        return pulumi.get(self, "blue_green_confirm")

    @blue_green_confirm.setter
    def blue_green_confirm(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "blue_green_confirm", value)

    @property
    @pulumi.getter(name="clusterName")
    def cluster_name(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "cluster_name")

    @cluster_name.setter
    def cluster_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "cluster_name", value)

    @property
    @pulumi.getter(name="defaultDomain")
    def default_domain(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "default_domain")

    @default_domain.setter
    def default_domain(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "default_domain", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter
    def environment(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        return pulumi.get(self, "environment")

    @environment.setter
    def environment(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "environment", value)

    @property
    @pulumi.getter(name="latestImage")
    def latest_image(self) -> Optional[pulumi.Input[bool]]:
        return pulumi.get(self, "latest_image")

    @latest_image.setter
    def latest_image(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "latest_image", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def services(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ApplicationServiceArgs']]]]:
        return pulumi.get(self, "services")

    @services.setter
    def services(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ApplicationServiceArgs']]]]):
        pulumi.set(self, "services", value)

    @property
    @pulumi.getter
    def template(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "template")

    @template.setter
    def template(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "template", value)

    @property
    @pulumi.getter
    def version(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "version")

    @version.setter
    def version(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "version", value)


class Application(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 blue_green: Optional[pulumi.Input[bool]] = None,
                 blue_green_confirm: Optional[pulumi.Input[bool]] = None,
                 cluster_name: Optional[pulumi.Input[str]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 environment: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 latest_image: Optional[pulumi.Input[bool]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 template: Optional[pulumi.Input[str]] = None,
                 version: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Create a Application resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ApplicationArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Create a Application resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param ApplicationArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ApplicationArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 blue_green: Optional[pulumi.Input[bool]] = None,
                 blue_green_confirm: Optional[pulumi.Input[bool]] = None,
                 cluster_name: Optional[pulumi.Input[str]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 environment: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 latest_image: Optional[pulumi.Input[bool]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 template: Optional[pulumi.Input[str]] = None,
                 version: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ApplicationArgs.__new__(ApplicationArgs)

            __props__.__dict__["blue_green"] = blue_green
            __props__.__dict__["blue_green_confirm"] = blue_green_confirm
            if cluster_name is None and not opts.urn:
                raise TypeError("Missing required property 'cluster_name'")
            __props__.__dict__["cluster_name"] = cluster_name
            __props__.__dict__["description"] = description
            __props__.__dict__["environment"] = environment
            __props__.__dict__["latest_image"] = latest_image
            __props__.__dict__["name"] = name
            if template is None and not opts.urn:
                raise TypeError("Missing required property 'template'")
            __props__.__dict__["template"] = template
            __props__.__dict__["version"] = version
            __props__.__dict__["default_domain"] = None
            __props__.__dict__["services"] = None
        super(Application, __self__).__init__(
            'alicloud:cs/application:Application',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            blue_green: Optional[pulumi.Input[bool]] = None,
            blue_green_confirm: Optional[pulumi.Input[bool]] = None,
            cluster_name: Optional[pulumi.Input[str]] = None,
            default_domain: Optional[pulumi.Input[str]] = None,
            description: Optional[pulumi.Input[str]] = None,
            environment: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            latest_image: Optional[pulumi.Input[bool]] = None,
            name: Optional[pulumi.Input[str]] = None,
            services: Optional[pulumi.Input[Sequence[pulumi.Input[Union['ApplicationServiceArgs', 'ApplicationServiceArgsDict']]]]] = None,
            template: Optional[pulumi.Input[str]] = None,
            version: Optional[pulumi.Input[str]] = None) -> 'Application':
        """
        Get an existing Application resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ApplicationState.__new__(_ApplicationState)

        __props__.__dict__["blue_green"] = blue_green
        __props__.__dict__["blue_green_confirm"] = blue_green_confirm
        __props__.__dict__["cluster_name"] = cluster_name
        __props__.__dict__["default_domain"] = default_domain
        __props__.__dict__["description"] = description
        __props__.__dict__["environment"] = environment
        __props__.__dict__["latest_image"] = latest_image
        __props__.__dict__["name"] = name
        __props__.__dict__["services"] = services
        __props__.__dict__["template"] = template
        __props__.__dict__["version"] = version
        return Application(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="blueGreen")
    def blue_green(self) -> pulumi.Output[Optional[bool]]:
        return pulumi.get(self, "blue_green")

    @property
    @pulumi.getter(name="blueGreenConfirm")
    def blue_green_confirm(self) -> pulumi.Output[Optional[bool]]:
        return pulumi.get(self, "blue_green_confirm")

    @property
    @pulumi.getter(name="clusterName")
    def cluster_name(self) -> pulumi.Output[str]:
        return pulumi.get(self, "cluster_name")

    @property
    @pulumi.getter(name="defaultDomain")
    def default_domain(self) -> pulumi.Output[str]:
        return pulumi.get(self, "default_domain")

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "description")

    @property
    @pulumi.getter
    def environment(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        return pulumi.get(self, "environment")

    @property
    @pulumi.getter(name="latestImage")
    def latest_image(self) -> pulumi.Output[Optional[bool]]:
        return pulumi.get(self, "latest_image")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def services(self) -> pulumi.Output[Sequence['outputs.ApplicationService']]:
        return pulumi.get(self, "services")

    @property
    @pulumi.getter
    def template(self) -> pulumi.Output[str]:
        return pulumi.get(self, "template")

    @property
    @pulumi.getter
    def version(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "version")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities

__all__ = ['Ipv6GatewayArgs', 'Ipv6Gateway']

@pulumi.input_type
class Ipv6GatewayArgs:
    def __init__(__self__, *,
                 vpc_id: pulumi.Input[str],
                 description: Optional[pulumi.Input[str]] = None,
                 ipv6_gateway_name: Optional[pulumi.Input[str]] = None,
                 resource_group_id: Optional[pulumi.Input[str]] = None,
                 spec: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None):
        """
        The set of arguments for constructing a Ipv6Gateway resource.
        :param pulumi.Input[str] vpc_id: The ID of the virtual private cloud (VPC) for which you want to create the IPv6 gateway.
        :param pulumi.Input[str] description: The description of the IPv6 gateway. The description must be 2 to 256 characters in length. It cannot start with http:// or https://.
        :param pulumi.Input[str] ipv6_gateway_name: The name of the IPv6 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, underscores (_), and hyphens (-). The name must start with a letter but cannot start with http:// or https://.
        :param pulumi.Input[str] resource_group_id: The ID of the resource group to which the instance belongs.
        :param pulumi.Input[str] spec: IPv6 gateways do not distinguish between specifications. This parameter is no longer used.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: The tags for the resource.
        """
        pulumi.set(__self__, "vpc_id", vpc_id)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if ipv6_gateway_name is not None:
            pulumi.set(__self__, "ipv6_gateway_name", ipv6_gateway_name)
        if resource_group_id is not None:
            pulumi.set(__self__, "resource_group_id", resource_group_id)
        if spec is not None:
            warnings.warn("""Field 'Spec' has been deprecated from provider version 1.205.0. IPv6 gateways do not distinguish between specifications. This parameter is no longer used.""", DeprecationWarning)
            pulumi.log.warn("""spec is deprecated: Field 'Spec' has been deprecated from provider version 1.205.0. IPv6 gateways do not distinguish between specifications. This parameter is no longer used.""")
        if spec is not None:
            pulumi.set(__self__, "spec", spec)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)

    @property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> pulumi.Input[str]:
        """
        The ID of the virtual private cloud (VPC) for which you want to create the IPv6 gateway.
        """
        return pulumi.get(self, "vpc_id")

    @vpc_id.setter
    def vpc_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "vpc_id", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the IPv6 gateway. The description must be 2 to 256 characters in length. It cannot start with http:// or https://.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="ipv6GatewayName")
    def ipv6_gateway_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the IPv6 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, underscores (_), and hyphens (-). The name must start with a letter but cannot start with http:// or https://.
        """
        return pulumi.get(self, "ipv6_gateway_name")

    @ipv6_gateway_name.setter
    def ipv6_gateway_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ipv6_gateway_name", value)

    @property
    @pulumi.getter(name="resourceGroupId")
    def resource_group_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the resource group to which the instance belongs.
        """
        return pulumi.get(self, "resource_group_id")

    @resource_group_id.setter
    def resource_group_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "resource_group_id", value)

    @property
    @pulumi.getter
    @_utilities.deprecated("""Field 'Spec' has been deprecated from provider version 1.205.0. IPv6 gateways do not distinguish between specifications. This parameter is no longer used.""")
    def spec(self) -> Optional[pulumi.Input[str]]:
        """
        IPv6 gateways do not distinguish between specifications. This parameter is no longer used.
        """
        return pulumi.get(self, "spec")

    @spec.setter
    def spec(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "spec", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        The tags for the resource.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)


@pulumi.input_type
class _Ipv6GatewayState:
    def __init__(__self__, *,
                 business_status: Optional[pulumi.Input[str]] = None,
                 create_time: Optional[pulumi.Input[str]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 expired_time: Optional[pulumi.Input[str]] = None,
                 instance_charge_type: Optional[pulumi.Input[str]] = None,
                 ipv6_gateway_id: Optional[pulumi.Input[str]] = None,
                 ipv6_gateway_name: Optional[pulumi.Input[str]] = None,
                 resource_group_id: Optional[pulumi.Input[str]] = None,
                 spec: Optional[pulumi.Input[str]] = None,
                 status: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 vpc_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Ipv6Gateway resources.
        :param pulumi.Input[str] business_status: The status of the IPv6 gateway.
        :param pulumi.Input[str] create_time: The creation time of the resource.
        :param pulumi.Input[str] description: The description of the IPv6 gateway. The description must be 2 to 256 characters in length. It cannot start with http:// or https://.
        :param pulumi.Input[str] expired_time: The expiration time of IPv6 gateway.
        :param pulumi.Input[str] instance_charge_type: The charge type of IPv6 gateway.
        :param pulumi.Input[str] ipv6_gateway_id: Resource primary key attribute field.
        :param pulumi.Input[str] ipv6_gateway_name: The name of the IPv6 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, underscores (_), and hyphens (-). The name must start with a letter but cannot start with http:// or https://.
        :param pulumi.Input[str] resource_group_id: The ID of the resource group to which the instance belongs.
        :param pulumi.Input[str] spec: IPv6 gateways do not distinguish between specifications. This parameter is no longer used.
        :param pulumi.Input[str] status: The status of the resource. Valid values: Available, Pending and Deleting.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: The tags for the resource.
        :param pulumi.Input[str] vpc_id: The ID of the virtual private cloud (VPC) for which you want to create the IPv6 gateway.
        """
        if business_status is not None:
            pulumi.set(__self__, "business_status", business_status)
        if create_time is not None:
            pulumi.set(__self__, "create_time", create_time)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if expired_time is not None:
            pulumi.set(__self__, "expired_time", expired_time)
        if instance_charge_type is not None:
            pulumi.set(__self__, "instance_charge_type", instance_charge_type)
        if ipv6_gateway_id is not None:
            pulumi.set(__self__, "ipv6_gateway_id", ipv6_gateway_id)
        if ipv6_gateway_name is not None:
            pulumi.set(__self__, "ipv6_gateway_name", ipv6_gateway_name)
        if resource_group_id is not None:
            pulumi.set(__self__, "resource_group_id", resource_group_id)
        if spec is not None:
            warnings.warn("""Field 'Spec' has been deprecated from provider version 1.205.0. IPv6 gateways do not distinguish between specifications. This parameter is no longer used.""", DeprecationWarning)
            pulumi.log.warn("""spec is deprecated: Field 'Spec' has been deprecated from provider version 1.205.0. IPv6 gateways do not distinguish between specifications. This parameter is no longer used.""")
        if spec is not None:
            pulumi.set(__self__, "spec", spec)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if tags is not None:
            pulumi.set(__self__, "tags", tags)
        if vpc_id is not None:
            pulumi.set(__self__, "vpc_id", vpc_id)

    @property
    @pulumi.getter(name="businessStatus")
    def business_status(self) -> Optional[pulumi.Input[str]]:
        """
        The status of the IPv6 gateway.
        """
        return pulumi.get(self, "business_status")

    @business_status.setter
    def business_status(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "business_status", value)

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> Optional[pulumi.Input[str]]:
        """
        The creation time of the resource.
        """
        return pulumi.get(self, "create_time")

    @create_time.setter
    def create_time(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "create_time", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        The description of the IPv6 gateway. The description must be 2 to 256 characters in length. It cannot start with http:// or https://.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="expiredTime")
    def expired_time(self) -> Optional[pulumi.Input[str]]:
        """
        The expiration time of IPv6 gateway.
        """
        return pulumi.get(self, "expired_time")

    @expired_time.setter
    def expired_time(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "expired_time", value)

    @property
    @pulumi.getter(name="instanceChargeType")
    def instance_charge_type(self) -> Optional[pulumi.Input[str]]:
        """
        The charge type of IPv6 gateway.
        """
        return pulumi.get(self, "instance_charge_type")

    @instance_charge_type.setter
    def instance_charge_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "instance_charge_type", value)

    @property
    @pulumi.getter(name="ipv6GatewayId")
    def ipv6_gateway_id(self) -> Optional[pulumi.Input[str]]:
        """
        Resource primary key attribute field.
        """
        return pulumi.get(self, "ipv6_gateway_id")

    @ipv6_gateway_id.setter
    def ipv6_gateway_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ipv6_gateway_id", value)

    @property
    @pulumi.getter(name="ipv6GatewayName")
    def ipv6_gateway_name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the IPv6 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, underscores (_), and hyphens (-). The name must start with a letter but cannot start with http:// or https://.
        """
        return pulumi.get(self, "ipv6_gateway_name")

    @ipv6_gateway_name.setter
    def ipv6_gateway_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ipv6_gateway_name", value)

    @property
    @pulumi.getter(name="resourceGroupId")
    def resource_group_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the resource group to which the instance belongs.
        """
        return pulumi.get(self, "resource_group_id")

    @resource_group_id.setter
    def resource_group_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "resource_group_id", value)

    @property
    @pulumi.getter
    @_utilities.deprecated("""Field 'Spec' has been deprecated from provider version 1.205.0. IPv6 gateways do not distinguish between specifications. This parameter is no longer used.""")
    def spec(self) -> Optional[pulumi.Input[str]]:
        """
        IPv6 gateways do not distinguish between specifications. This parameter is no longer used.
        """
        return pulumi.get(self, "spec")

    @spec.setter
    def spec(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "spec", value)

    @property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[str]]:
        """
        The status of the resource. Valid values: Available, Pending and Deleting.
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "status", value)

    @property
    @pulumi.getter
    def tags(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        The tags for the resource.
        """
        return pulumi.get(self, "tags")

    @tags.setter
    def tags(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "tags", value)

    @property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the virtual private cloud (VPC) for which you want to create the IPv6 gateway.
        """
        return pulumi.get(self, "vpc_id")

    @vpc_id.setter
    def vpc_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "vpc_id", value)


class Ipv6Gateway(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 ipv6_gateway_name: Optional[pulumi.Input[str]] = None,
                 resource_group_id: Optional[pulumi.Input[str]] = None,
                 spec: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 vpc_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Provides a Vpc Ipv6 Gateway resource. Gateway Based on Internet Protocol Version 6.

        For information about Vpc Ipv6 Gateway and how to use it, see [What is Ipv6 Gateway](https://www.alibabacloud.com/help/en/virtual-private-cloud/latest/createipv6gateway).

        > **NOTE:** Available in v1.142.0+.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        config = pulumi.Config()
        name = config.get("name")
        if name is None:
            name = "tf-testacc-example"
        default_vpc = alicloud.vpc.Network("defaultVpc",
            description="tf-testacc",
            enable_ipv6=True)
        default_rg = alicloud.resourcemanager.ResourceGroup("defaultRg",
            display_name="tf-testacc-ipv6gateway503",
            resource_group_name=f"{name}1")
        change_rg = alicloud.resourcemanager.ResourceGroup("changeRg",
            display_name="tf-testacc-ipv6gateway311",
            resource_group_name=f"{name}2")
        default = alicloud.vpc.Ipv6Gateway("default",
            description="test",
            ipv6_gateway_name=name,
            vpc_id=default_vpc.id,
            resource_group_id=default_rg.id)
        ```

        ## Import

        Vpc Ipv6 Gateway can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:vpc/ipv6Gateway:Ipv6Gateway example <id>
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: The description of the IPv6 gateway. The description must be 2 to 256 characters in length. It cannot start with http:// or https://.
        :param pulumi.Input[str] ipv6_gateway_name: The name of the IPv6 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, underscores (_), and hyphens (-). The name must start with a letter but cannot start with http:// or https://.
        :param pulumi.Input[str] resource_group_id: The ID of the resource group to which the instance belongs.
        :param pulumi.Input[str] spec: IPv6 gateways do not distinguish between specifications. This parameter is no longer used.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: The tags for the resource.
        :param pulumi.Input[str] vpc_id: The ID of the virtual private cloud (VPC) for which you want to create the IPv6 gateway.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Ipv6GatewayArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a Vpc Ipv6 Gateway resource. Gateway Based on Internet Protocol Version 6.

        For information about Vpc Ipv6 Gateway and how to use it, see [What is Ipv6 Gateway](https://www.alibabacloud.com/help/en/virtual-private-cloud/latest/createipv6gateway).

        > **NOTE:** Available in v1.142.0+.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        config = pulumi.Config()
        name = config.get("name")
        if name is None:
            name = "tf-testacc-example"
        default_vpc = alicloud.vpc.Network("defaultVpc",
            description="tf-testacc",
            enable_ipv6=True)
        default_rg = alicloud.resourcemanager.ResourceGroup("defaultRg",
            display_name="tf-testacc-ipv6gateway503",
            resource_group_name=f"{name}1")
        change_rg = alicloud.resourcemanager.ResourceGroup("changeRg",
            display_name="tf-testacc-ipv6gateway311",
            resource_group_name=f"{name}2")
        default = alicloud.vpc.Ipv6Gateway("default",
            description="test",
            ipv6_gateway_name=name,
            vpc_id=default_vpc.id,
            resource_group_id=default_rg.id)
        ```

        ## Import

        Vpc Ipv6 Gateway can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:vpc/ipv6Gateway:Ipv6Gateway example <id>
        ```

        :param str resource_name: The name of the resource.
        :param Ipv6GatewayArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(Ipv6GatewayArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 ipv6_gateway_name: Optional[pulumi.Input[str]] = None,
                 resource_group_id: Optional[pulumi.Input[str]] = None,
                 spec: Optional[pulumi.Input[str]] = None,
                 tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 vpc_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = Ipv6GatewayArgs.__new__(Ipv6GatewayArgs)

            __props__.__dict__["description"] = description
            __props__.__dict__["ipv6_gateway_name"] = ipv6_gateway_name
            __props__.__dict__["resource_group_id"] = resource_group_id
            __props__.__dict__["spec"] = spec
            __props__.__dict__["tags"] = tags
            if vpc_id is None and not opts.urn:
                raise TypeError("Missing required property 'vpc_id'")
            __props__.__dict__["vpc_id"] = vpc_id
            __props__.__dict__["business_status"] = None
            __props__.__dict__["create_time"] = None
            __props__.__dict__["expired_time"] = None
            __props__.__dict__["instance_charge_type"] = None
            __props__.__dict__["ipv6_gateway_id"] = None
            __props__.__dict__["status"] = None
        super(Ipv6Gateway, __self__).__init__(
            'alicloud:vpc/ipv6Gateway:Ipv6Gateway',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            business_status: Optional[pulumi.Input[str]] = None,
            create_time: Optional[pulumi.Input[str]] = None,
            description: Optional[pulumi.Input[str]] = None,
            expired_time: Optional[pulumi.Input[str]] = None,
            instance_charge_type: Optional[pulumi.Input[str]] = None,
            ipv6_gateway_id: Optional[pulumi.Input[str]] = None,
            ipv6_gateway_name: Optional[pulumi.Input[str]] = None,
            resource_group_id: Optional[pulumi.Input[str]] = None,
            spec: Optional[pulumi.Input[str]] = None,
            status: Optional[pulumi.Input[str]] = None,
            tags: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
            vpc_id: Optional[pulumi.Input[str]] = None) -> 'Ipv6Gateway':
        """
        Get an existing Ipv6Gateway resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] business_status: The status of the IPv6 gateway.
        :param pulumi.Input[str] create_time: The creation time of the resource.
        :param pulumi.Input[str] description: The description of the IPv6 gateway. The description must be 2 to 256 characters in length. It cannot start with http:// or https://.
        :param pulumi.Input[str] expired_time: The expiration time of IPv6 gateway.
        :param pulumi.Input[str] instance_charge_type: The charge type of IPv6 gateway.
        :param pulumi.Input[str] ipv6_gateway_id: Resource primary key attribute field.
        :param pulumi.Input[str] ipv6_gateway_name: The name of the IPv6 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, underscores (_), and hyphens (-). The name must start with a letter but cannot start with http:// or https://.
        :param pulumi.Input[str] resource_group_id: The ID of the resource group to which the instance belongs.
        :param pulumi.Input[str] spec: IPv6 gateways do not distinguish between specifications. This parameter is no longer used.
        :param pulumi.Input[str] status: The status of the resource. Valid values: Available, Pending and Deleting.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] tags: The tags for the resource.
        :param pulumi.Input[str] vpc_id: The ID of the virtual private cloud (VPC) for which you want to create the IPv6 gateway.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _Ipv6GatewayState.__new__(_Ipv6GatewayState)

        __props__.__dict__["business_status"] = business_status
        __props__.__dict__["create_time"] = create_time
        __props__.__dict__["description"] = description
        __props__.__dict__["expired_time"] = expired_time
        __props__.__dict__["instance_charge_type"] = instance_charge_type
        __props__.__dict__["ipv6_gateway_id"] = ipv6_gateway_id
        __props__.__dict__["ipv6_gateway_name"] = ipv6_gateway_name
        __props__.__dict__["resource_group_id"] = resource_group_id
        __props__.__dict__["spec"] = spec
        __props__.__dict__["status"] = status
        __props__.__dict__["tags"] = tags
        __props__.__dict__["vpc_id"] = vpc_id
        return Ipv6Gateway(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="businessStatus")
    def business_status(self) -> pulumi.Output[str]:
        """
        The status of the IPv6 gateway.
        """
        return pulumi.get(self, "business_status")

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> pulumi.Output[str]:
        """
        The creation time of the resource.
        """
        return pulumi.get(self, "create_time")

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[str]]:
        """
        The description of the IPv6 gateway. The description must be 2 to 256 characters in length. It cannot start with http:// or https://.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="expiredTime")
    def expired_time(self) -> pulumi.Output[str]:
        """
        The expiration time of IPv6 gateway.
        """
        return pulumi.get(self, "expired_time")

    @property
    @pulumi.getter(name="instanceChargeType")
    def instance_charge_type(self) -> pulumi.Output[str]:
        """
        The charge type of IPv6 gateway.
        """
        return pulumi.get(self, "instance_charge_type")

    @property
    @pulumi.getter(name="ipv6GatewayId")
    def ipv6_gateway_id(self) -> pulumi.Output[str]:
        """
        Resource primary key attribute field.
        """
        return pulumi.get(self, "ipv6_gateway_id")

    @property
    @pulumi.getter(name="ipv6GatewayName")
    def ipv6_gateway_name(self) -> pulumi.Output[Optional[str]]:
        """
        The name of the IPv6 gateway. The name must be 2 to 128 characters in length, and can contain letters, digits, underscores (_), and hyphens (-). The name must start with a letter but cannot start with http:// or https://.
        """
        return pulumi.get(self, "ipv6_gateway_name")

    @property
    @pulumi.getter(name="resourceGroupId")
    def resource_group_id(self) -> pulumi.Output[str]:
        """
        The ID of the resource group to which the instance belongs.
        """
        return pulumi.get(self, "resource_group_id")

    @property
    @pulumi.getter
    @_utilities.deprecated("""Field 'Spec' has been deprecated from provider version 1.205.0. IPv6 gateways do not distinguish between specifications. This parameter is no longer used.""")
    def spec(self) -> pulumi.Output[str]:
        """
        IPv6 gateways do not distinguish between specifications. This parameter is no longer used.
        """
        return pulumi.get(self, "spec")

    @property
    @pulumi.getter
    def status(self) -> pulumi.Output[str]:
        """
        The status of the resource. Valid values: Available, Pending and Deleting.
        """
        return pulumi.get(self, "status")

    @property
    @pulumi.getter
    def tags(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        The tags for the resource.
        """
        return pulumi.get(self, "tags")

    @property
    @pulumi.getter(name="vpcId")
    def vpc_id(self) -> pulumi.Output[str]:
        """
        The ID of the virtual private cloud (VPC) for which you want to create the IPv6 gateway.
        """
        return pulumi.get(self, "vpc_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['SasTrailArgs', 'SasTrail']

@pulumi.input_type
class SasTrailArgs:
    def __init__(__self__):
        """
        The set of arguments for constructing a SasTrail resource.
        """
        pass


@pulumi.input_type
class _SasTrailState:
    def __init__(__self__, *,
                 create_time: Optional[pulumi.Input[int]] = None,
                 service_trail: Optional[pulumi.Input['SasTrailServiceTrailArgs']] = None):
        """
        Input properties used for looking up and filtering SasTrail resources.
        :param pulumi.Input[int] create_time: The service trace creation timestamp, in milliseconds.
        :param pulumi.Input['SasTrailServiceTrailArgs'] service_trail: Service trace configuration information.
        """
        if create_time is not None:
            pulumi.set(__self__, "create_time", create_time)
        if service_trail is not None:
            pulumi.set(__self__, "service_trail", service_trail)

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> Optional[pulumi.Input[int]]:
        """
        The service trace creation timestamp, in milliseconds.
        """
        return pulumi.get(self, "create_time")

    @create_time.setter
    def create_time(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "create_time", value)

    @property
    @pulumi.getter(name="serviceTrail")
    def service_trail(self) -> Optional[pulumi.Input['SasTrailServiceTrailArgs']]:
        """
        Service trace configuration information.
        """
        return pulumi.get(self, "service_trail")

    @service_trail.setter
    def service_trail(self, value: Optional[pulumi.Input['SasTrailServiceTrailArgs']]):
        pulumi.set(self, "service_trail", value)


class SasTrail(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 __props__=None):
        """
        Provides a Threat Detection Sas Trail resource.

        For information about Threat Detection Sas Trail and how to use it, see [What is Sas Trail](https://www.alibabacloud.com/help/zh/security-center/developer-reference/api-sas-2018-12-03-createservicetrail).

        > **NOTE:** Available since v1.212.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        config = pulumi.Config()
        name = config.get("name")
        if name is None:
            name = "terraform-example"
        default = alicloud.threatdetection.SasTrail("default")
        ```

        ## Import

        Threat Detection Sas Trail can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:threatdetection/sasTrail:SasTrail example
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[SasTrailArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Provides a Threat Detection Sas Trail resource.

        For information about Threat Detection Sas Trail and how to use it, see [What is Sas Trail](https://www.alibabacloud.com/help/zh/security-center/developer-reference/api-sas-2018-12-03-createservicetrail).

        > **NOTE:** Available since v1.212.0.

        ## Example Usage

        Basic Usage

        ```python
        import pulumi
        import pulumi_alicloud as alicloud

        config = pulumi.Config()
        name = config.get("name")
        if name is None:
            name = "terraform-example"
        default = alicloud.threatdetection.SasTrail("default")
        ```

        ## Import

        Threat Detection Sas Trail can be imported using the id, e.g.

        ```sh
        $ pulumi import alicloud:threatdetection/sasTrail:SasTrail example
        ```

        :param str resource_name: The name of the resource.
        :param SasTrailArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(SasTrailArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = SasTrailArgs.__new__(SasTrailArgs)

            __props__.__dict__["create_time"] = None
            __props__.__dict__["service_trail"] = None
        super(SasTrail, __self__).__init__(
            'alicloud:threatdetection/sasTrail:SasTrail',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            create_time: Optional[pulumi.Input[int]] = None,
            service_trail: Optional[pulumi.Input[Union['SasTrailServiceTrailArgs', 'SasTrailServiceTrailArgsDict']]] = None) -> 'SasTrail':
        """
        Get an existing SasTrail resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[int] create_time: The service trace creation timestamp, in milliseconds.
        :param pulumi.Input[Union['SasTrailServiceTrailArgs', 'SasTrailServiceTrailArgsDict']] service_trail: Service trace configuration information.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _SasTrailState.__new__(_SasTrailState)

        __props__.__dict__["create_time"] = create_time
        __props__.__dict__["service_trail"] = service_trail
        return SasTrail(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="createTime")
    def create_time(self) -> pulumi.Output[int]:
        """
        The service trace creation timestamp, in milliseconds.
        """
        return pulumi.get(self, "create_time")

    @property
    @pulumi.getter(name="serviceTrail")
    def service_trail(self) -> pulumi.Output['outputs.SasTrailServiceTrail']:
        """
        Service trace configuration information.
        """
        return pulumi.get(self, "service_trail")

